/*************************************************************************
 *
 *   Program:  PicoM_Utilities.h		Version: 1.00
 *
 *   Author:  SS						Date:  Dec 19, 2014
 *
 *   This program contains the PaellaM Utility functions for  serial io, etc. used by
 *   various files making up this project.
 *
 *   This project is licensed under the MIT license.  You can modify, enhance, etc anyway you like.
 *
 *   SOC Robotics, Inc is not liable in anyway for the use of this software and assumes no responsibility 
 *   for it's use.  No warranties of any kind are provided.
 *
 *   Copyright 2014, SOC Robotics, Inc.  
 *   There maybe code from other parties in this application  If we don't mention them by name thank you
 *   and if we should please let us know and we will include your name and attribution.
 *
 *************************************************************************/

/**************************************************************************
	Utility functions
**************************************************************************/
//#include "hw_config.h"

#include "stdint.h"
#include "stdbool.h"
#include "stdarg.h"
#include "stdio.h"
#include "stdlib.h"
#include "math.h"
#include "string.h"
#include "stddef.h"
#include "ctype.h"

#include "PaellaM_Defines.h"
#include "PaellaM_Externals.h"
#include "NRF24L01.h"
#include "FXOS8700.h"
#include "MMA8653.h"
#include "SI7013.h"
#include "st7735.h"
#include "colors.h"



// FXOS8700 variables
//typedef struct
//{
//	short int xa,ya,za;
//	short int xm,ym,zm;
//} FXOSRawData;

// FXOS8700 variables
//typedef struct
//{
//	float ax,ay,az;
//	float mx,my,mz;
//} FXOS_F;
extern volatile FXOSRawData fxosRawData;
extern volatile struct sid {
	unsigned int rawtemperature,rawhumidity;
	float temperature,humidity;
} si7013;

extern FXOS_F dip[];
extern FXOS_F pip[];
extern FXOS_F mcp[];

/**************************************************************************
   Begin utility functions
**************************************************************************/
char *itoa(int val, int base)
{
	static char buf[32] = {0};

	int i = 30;

	for(; val && i ; --i, val /= base)

		buf[i] = "0123456789abcdef"[val % base];

	return &buf[i+1];
}

/**************************************************************************
	ftoa Function - lean function to reduce printf float print footprint

	precision - number of fractional digits
	numchars - width of float output - if numchars>output number shifted
	                                   to the right and padded with spaces
**************************************************************************/
char *ftoa(volatile float num,volatile int precision,volatile int numchars)
{
	volatile char signchar=' ';
	volatile int intpart;
	volatile long intdecimal;
	volatile int i,shortage;
	volatile float decimal_part;
	volatile char decimal[20],str[20];

	if(num<0) signchar = '-';
	num = fabs(num);
	intpart = num;
	memset(str, 0x0, 20);
	memset(decimal, 0x0, 20);
	str[0] = signchar;
	strcpy(decimal,itoa(intpart,10));

	strcat(str, decimal);
	strcat(str,".");

	decimal_part = num - intpart;
	if(precision>7) precision = 7;
	for(i=0; i<precision; i++) decimal_part *= 10.0;
	intdecimal = decimal_part;
	//printf(" intdecimal=%d ",intdecimal);

	strcpy(decimal,itoa(intdecimal,10));
	//printf(" %s ",decimal);

	for(i=0; i<(precision - strlen(decimal)); i++)	strcat(str, "0");
	strcat(str, decimal);
	// If string is too long shorten it up
	//printf(" str=%s l=%d",str,strlen(str));
	if(strlen(str)>=numchars) {
		str[numchars] = 0;
		//printf(" str=%s ",str);
		//printf(" * ");
		return str;
		}
	// If string is too short shift all characters right and fill front with space
	else {
		shortage = numchars - strlen(str);
		memset(decimal, 0x0, 20);
		for(i=0; i<shortage; i++) decimal[i] = ' ';
		strcat(decimal,str);
		strcpy(str,decimal);
		//printf(" # ");
		return str;
		}
}

/***************************************************
     TWI functions
****************************************************/
// Wrapper for Atmel ASF TWI Write to support different I2C slaves
volatile unsigned char TWI_Write(volatile  unsigned char address, volatile unsigned char *msg, volatile unsigned char msgSize )
{
	int i;

	for(i=0; i<msgSize; i++)  messageBuf[i] = msg[i];

	return I2C_WriteBytes(I2C1,address,msgSize);
}

// Repeated start is required to communicate with FXOS8700 6DOF Sensor and MMA8653F 3DOF Sensor
volatile unsigned char TWI_Write_Repeat( volatile unsigned char address, volatile unsigned char *msg, volatile unsigned char msgSize )
{
	//return  twim_write_repeat(&TWI_MASTER, msg,	msgSize, address, false);
	return  1;
}

// Wrapper for Atmel TWI Read to support different I2C slaves
volatile unsigned char TWI_Start_Read( volatile unsigned char address, volatile unsigned char msgSize )
{
	return I2C_ReadBytes(I2C1,address,msgSize);
}

// Simple function to check is slave exists
unsigned char  twim_probe(I2C_TypeDef* I2Cx,uint32_t chip_addr)
{
	return STATUS_OK;
}

unsigned char twim_chained_transfer (I2C_TypeDef* I2Cx,unsigned char *first,
		unsigned char *second,bool tenbit)
{
	return 1;
}

/***************************************************
   Timer Functions
****************************************************/
unsigned long StartTimer(void)
{
	return mseccounter;
}

float ElapsedTimesec(unsigned long starttime)
{
	return (float)(mseccounter-starttime)/1000.0;
}

long ElapsedTimeusec(void)
{
	TIM7_Stop();
	return (TIM_GetCounter(TIM7)+(long)usectimertally*65535);
}

/***************************************************
   EEPROM functions - allows int and float read/writes
   to EEPROM
****************************************************/
int EEPROM_Read_Int(unsigned char ptr)
{
	union {
	    unsigned char tempbuf[2];
		int outint;
		} u;
	
	u.tempbuf[0] = nvm_eeprom_read_byte(ptr++);
	u.tempbuf[1] = nvm_eeprom_read_byte(ptr);
	return u.outint;
}

void EEPROM_Write_Int(unsigned char ptr,int value)
{
	union {
	    unsigned char tempbuf[2];
		int outint;
		} u;

    u.outint = value;
	nvm_eeprom_write_byte(ptr++,u.tempbuf[0]);
	nvm_eeprom_write_byte(ptr,u.tempbuf[1]);
}

float EEPROM_Read_Float(unsigned char ptr)
{
	union {
	    unsigned char tempbuf[8];
		float outfloat;
		} u;

	u.tempbuf[0] = nvm_eeprom_read_byte(ptr++);
	u.tempbuf[1] = nvm_eeprom_read_byte(ptr++);
	u.tempbuf[2] = nvm_eeprom_read_byte(ptr++);
	u.tempbuf[3] = nvm_eeprom_read_byte(ptr);
	return u.outfloat;
}

void EEPROM_Write_Float(unsigned char ptr,float value)
{
	union {
	    unsigned char tempbuf[8];
		float outfloat;
		} u;

	u.outfloat = value;
	nvm_eeprom_write_byte(ptr++,u.tempbuf[0]);
	nvm_eeprom_write_byte(ptr++,u.tempbuf[1]);
	nvm_eeprom_write_byte(ptr++,u.tempbuf[2]);
	nvm_eeprom_write_byte(ptr,u.tempbuf[3]);
}

// Load Pico parameters
void EEPROM_LoadParameters(void)
{
	volatile int ptr=3;

	channel_no = nvm_eeprom_read_byte(ptr++);
	rf_payload_size = nvm_eeprom_read_byte(ptr++);
	device_id = nvm_eeprom_read_byte(ptr++);
	rf_device_class = nvm_eeprom_read_byte(ptr++);
}

// Remember EEPROM pages are 32 bytes
void EEPROM_StoreParameters(void)
{
	volatile int ptr=3;

	Puts("\n\rStore parameters to EEPROM\n\r");
    nvm_eeprom_write_byte(ptr++,channel_no);
    nvm_eeprom_write_byte(ptr++,rf_payload_size);
    nvm_eeprom_write_byte(ptr++,device_id);
    nvm_eeprom_write_byte(ptr++,rf_device_class);
}

void EEPROM_WriteSignature(void)
{
	nvm_eeprom_write_byte(0,'S');
	nvm_eeprom_write_byte(1,'O');
	nvm_eeprom_write_byte(2,'C');
}

// If signature exists use EEPROM parameters otherwise write program
// default parameters
int EEPROM_CheckSignature(void)
{
	int statevalue=0;
	
	if(nvm_eeprom_read_byte(0)!='S') statevalue = 1;
	if(nvm_eeprom_read_byte(1)!='O') statevalue = 1;
	if(nvm_eeprom_read_byte(2)!='C') statevalue = 1;	
	return statevalue;
}

// Dump BMP180, MPU9150 and RF24L01 parameters
void EEPROM_Printf(void)
{
	Puts("\n\rEEPROM Contents:\n\r");
	//printf("Signature=%c%c%c %x\n\r",nvm_eeprom_read_byte(0),nvm_eeprom_read_byte(1),nvm_eeprom_read_byte(2),nvm_eeprom_read_byte(0));
	//printf("Channel No=%d\n\r",nvm_eeprom_read_byte(3));
	//printf("Payload size=%d\n\r",nvm_eeprom_read_byte(4));
	//printf("Device id=%d\n\r",nvm_eeprom_read_byte(5));
	//printf("Device class=%c %x\n\r",nvm_eeprom_read_byte(6),nvm_eeprom_read_byte(6));
	//printf("TADDR = %c %c %c %c %d\n\r",TADDR[0],TADDR[1],TADDR[2],TADDR[3],TADDR[4]);
}

//************************************************
// BMP180 Barometer Functions
//************************************************
unsigned char Read_BMP180_EEPROM(unsigned char eepromaddress)
{
	messageBuf[0] = eepromaddress;
    TWI_Write(BMP180TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	TWI_Start_Read(BMP180TWIADDRESS,1);
	return messageBuf[0];
}

short int Read_BMP180_EEPROM_Signed(unsigned char eepromaddress)
{
	union {
	    unsigned char tempbuf[2];
		short int outint;
		} u;
	
	u.outint = 0;
	u.tempbuf[1] = Read_BMP180_EEPROM(eepromaddress++);
	u.tempbuf[0] = Read_BMP180_EEPROM(eepromaddress);
    return u.outint;  
}

unsigned short int Read_BMP180_EEPROM_UnSigned(unsigned char eepromaddress)
{
	union {
	    unsigned char tempbuf[2];
		unsigned short int outint;
		} u;
	
	u.outint = 0;
	u.tempbuf[1] = Read_BMP180_EEPROM(eepromaddress++);
	u.tempbuf[0] = Read_BMP180_EEPROM(eepromaddress);
    return u.outint;  
}

// Load chip parameters required to compute temperature and pressure
void Load_BMP180_Parameters(void)
{
 	 bmpac1=bmpac2=bmpac3=bmpb1=bmpb2=bmpmb=bmpmc=bmpmd=0;
	 bmpac4=bmpac5=bmpac6=0;

 	 // Read signed BMP180 Parameters
	 bmpac1 = Read_BMP180_EEPROM_Signed(0xaa);
	 bmpac2 = Read_BMP180_EEPROM_Signed(0xac);
	 bmpac3 = Read_BMP180_EEPROM_Signed(0xae);
	 bmpb1 = Read_BMP180_EEPROM_Signed(0xb6);
	 bmpb2 = Read_BMP180_EEPROM_Signed(0xb8);
	 bmpmb = Read_BMP180_EEPROM_Signed(0xba);
	 bmpmc = Read_BMP180_EEPROM_Signed(0xbc);
	 bmpmd = Read_BMP180_EEPROM_Signed(0xbe);

	 // Dump registers - helps with diagnostics
	 if(0) {
/*	     printf("\n\rbmp180 Calibration Parameters\n\r");
	     printf("bmpac1=%d",bmpac1);
	     printf(" %x\n\r",bmpac1);
	     printf("bmpac2=%d",bmpac2);
	     printf(" %x\n\r",bmpac2);
	     printf("bmpac3=%d",bmpac3);
	     printf(" %x\n\r",bmpac3);
	     printf("bmpb1=%d",bmpb1);
	     printf(" %x\n\r",bmpb1);
	     printf("bmpb2=%d",bmpb2);
	     printf(" %x\n\r",bmpb2);
	     printf("bmpmb=%d",bmpmb);
	     printf(" %x\n\r",bmpmb);
	     printf("bmpmc=%d",bmpmc);
	     printf(" %x\n\r",bmpmc);
	     printf("bmpmd=%d",bmpmd);
	     printf(" %x\n\r",bmpmd);
*/
		 }

	 // Read unsigned BMP180 parameters
	 bmpac4 = Read_BMP180_EEPROM_UnSigned(0xb0);
	 bmpac5 = Read_BMP180_EEPROM_UnSigned(0xb2);
	 bmpac6 = Read_BMP180_EEPROM_UnSigned(0xb4);

	 if(0) {
/*	     printf("bmpac4=%u",bmpac4);
	     printf(" %x\n\r",bmpac4);
	     printf("bmpac5=%u",bmpac5);
	     printf(" %x\n\r",bmpac5);
	     printf("bmpac6=%u",bmpac6);
	     printf(" %x\n\r",bmpac6);
*/
		 }
}

// Barometer updates are slower than other mems sensors so rather than wait 
// for data to become available this functions starts the process and another
// retrieves the data
void Start_Read_BMP180_Pressure(void)
{
    //printf("\n\rRead barometric pressure\n\r");
	messageBuf[0] = 0xf4;
	messageBuf[1] = 0xb4;
    TWI_Write(BMP180TWIADDRESS,messageBuf,2);
}

unsigned short int Finish_Read_BMP180_Pressure(void)
{
	union {
	    unsigned char tempbuf[2];
		unsigned short int outint;
		} u;
	
	u.outint = 0;
	messageBuf[0] = 0xf6;
    TWI_Write(BMP180TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(BMP180TWIADDRESS,2);
	u.tempbuf[0] = messageBuf[1];
	u.tempbuf[1] = messageBuf[0];
	//printf("\n\rmb[0]= %x ",messageBuf[0]);
	//printf("mb[1]= %x ",messageBuf[1]);
	//printf("BMP Pressure raw=%d\n\r",u.outint);
	
    return u.outint;
}


unsigned short int Read_BMP180_Pressure(void)
{
	union {
	    unsigned char tempbuf[2];
		unsigned short int outint;
		} u;
	
	u.outint = 0;
    //printf("\n\rRead barometric pressure\n\r");
	messageBuf[0] = 0xf4;
	messageBuf[1] = 0xb4;
    TWI_Write(BMP180TWIADDRESS,messageBuf,2);
	delay_ms(28);
	messageBuf[0] = 0xf6;
    TWI_Write(BMP180TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(BMP180TWIADDRESS,2);
    //TWI_Get_Data_From_Transceiver(messageBuf,2);
	u.tempbuf[0] = messageBuf[1];
	u.tempbuf[1] = messageBuf[0];
	//printf("\n\rpress mb[0]= %x ",messageBuf[0]);
	//printf("mb[1]= %x ",messageBuf[1]);
	//printf("%x\n\r",u.outint);
	//printf("p=u.outint=%ld\n\r",u.outint);
	//printf("BMP Pressure raw in call=%d\n\r",u.outint);
	
    return u.outint;
}

float Finish_Read_BMP180_Pressurefloat(void)
{
	volatile unsigned int BMPpressure;
	volatile long x1,x2;
	volatile long b6,x3,b3,p;
	volatile unsigned long b4,b7;
	volatile float compensatedpressure;
	
	// Get raw pressure and compensate
	BMPpressure = Finish_Read_BMP180_Pressure();
	ibmppressure = BMPpressure;  // used in data logger routine
	
	b6 = b5 - 4000;
	x1 = ((long)bmpb2*((b6*b6)/4096))/2048;
	x2 = ((long)bmpac2*b6)/2048;
	x3 = x1 + x2;
	b3 = ((((long)bmpac1*4+x3)<<0)+2)/4;
	x1 = ((long)bmpac3*b6)/8192;
	x2 = ((long)bmpb1*((b6*b6)/4096))/65536;
	x3 = ((x1+x2)+2)/4;
	b4 = ((long)bmpac4*(unsigned long)(x3+32768))/32768;
	b7 = ((unsigned long)BMPpressure-b3)*((unsigned long)50000>>0);
	if(b7<0x80000000) p = (b7*2)/b4;
	else p = (b7/b4)*2;
	x1 = (p/256)*(p/256);
	x1 = (x1*3038)/65536;
	x2 = (-7357*p)/65536;
	p = p + (x1+x2+3791)/16;
	
	compensatedpressure = (float)p*0.00014504;
	return compensatedpressure;
}

float Read_BMP180_pressurefloat(char mode)
{	 
    volatile unsigned short int BMPpressure,BMPtemperature;
	volatile long x1,x2,temperatureinC,b5;
	volatile long b6,x3,b3,p;
	volatile unsigned long b4,b7;
	volatile float compensatedpressure;
	
	BMPpressure = BMPtemperature = 0x00;
	//Diagnostic constants
	//bmpac1 = 408; bmpac2 = -72; bmpac3 = -14383; bmpac4 = 32741;
	//bmpac5 = 32757; bmpac6 = 23151; bmpb1 = 6190; bmpb2 = 4;
	//bmpmb = -32768; bmpmc = -8711; bmpmd = 2868;
	
	// Get raw temperature and compensate
 	BMPtemperature = Read_BMP180_Temperature();
	//printf("bmptemp = %d \n\r",BMPtemperature);
	x1 = ((unsigned long)(BMPtemperature - (long)bmpac6) * (long)bmpac5)/(long)32768;
	x2 = ((long)bmpmc*2048)/(x1+(long)bmpmd);
	b5 = x1 + x2;
	temperatureinC = (b5 + 8)/16;
	temperature = temperatureinC/10.0;
	//printf("b5=%d\n\r",b5);
	
	// Get raw pressure and compensate
	BMPpressure = Read_BMP180_Pressure();
	//printf("BMP Pressure raw=%d\n\r",BMPpressure);

//	BMPpressure = 23843;
	
	b6 = b5 - 4000;
	//printf("b6 = %d\n\r",b6);
	x1 = ((long)bmpb2*((b6*b6)/4096))/2048;
	//printf("x1 = %d\n\r",x1);
	x2 = ((long)bmpac2*b6)/2048;
	//printf("x2 = %d\n\r",x2);
	x3 = x1 + x2;
	//printf("x3 = %d\n\r",x3);
	b3 = ((((long)bmpac1*4+x3)<<0)+2)/4;
	//printf("b3 = %d\n\r",b3);
	x1 = ((long)bmpac3*b6)/8192;
	//printf("x1 = %d\n\r",x1);
	x2 = ((long)bmpb1*((b6*b6)/4096))/65536;
	//printf("x2 = %d\n\r",x2);
	x3 = ((x1+x2)+2)/4;
	//printf("x3 = %d\n\r",x3);
	b4 = ((long)bmpac4*(unsigned long)(x3+32768))/32768;
	//printf("b4 = %u\n\r",b4);
	b7 = ((unsigned long)BMPpressure-b3)*((unsigned long)50000>>0);
	//printf("b7 = %lu\n\r",b7);
	if(b7<0x80000000) p = (b7*2)/b4;
	else p = (b7/b4)*2;
	//printf("p = %ld\n\r",p);
	x1 = (p/256)*(p/256);
	//printf("x1 = %ld\n\r",x1);
	x1 = (x1*3038)/65536;
	//printf("x1 = %d\n\r",x1);
	x2 = (-7357*p)/65536;
	//printf("x2 = %d\n\r",x2);
	p = p + (x1+x2+3791)/16;
	
// 	if(mode==VERBOSE) printf("\rBMP085 temperature raw = %u    \n\r",BMPtemperature);
// 	printf("\rBMP085 temperature (C)= %.2f    \n\r",(float)temperatureinC/10.0);
// 	if(mode==VERBOSE) printf("\rBMP085 pressure raw = %u    \n\r",BMPpressure);
// 	if(mode==VERBOSE) printf("\rBMP085 pressure (p) = %ld    \n\r",p);	
// 	printf("\rBMP085 pressure (psi) = %.4f    \n\r",(float)p*0.00014504);	
	compensatedpressure = (float)p*0.00014504;
	return compensatedpressure;
}


void Start_Read_BMP180_Temperature(void)
{
    //printf("\n\rRead temperature\n\r");
	messageBuf[0] = 0xf4;
	messageBuf[1] = 0x2e;
    TWI_Write(BMP180TWIADDRESS,messageBuf,2);
}

unsigned short int Finish_Read_BMP180_Temperature(void)
{
	union {
	    unsigned char tempbuf[2];
		unsigned short int outint;
		} u;
	
	u.outint = 0;
	messageBuf[0] = 0xf6;
    TWI_Write(BMP180TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(BMP180TWIADDRESS,2);
	u.tempbuf[0] = messageBuf[1];
	u.tempbuf[1] = messageBuf[0];
	//printf("mb[0]= %x\n\r",messageBuf[0]);
	//printf("mb[1]= %x\n\r",messageBuf[1]);
	
    return u.outint;
}


unsigned short int Read_BMP180_Temperature(void)
{
	union {
	    unsigned char tempbuf[2];
		unsigned short int outint;
		} u;
	
	u.outint = 0;
    //printf("\n\rRead temperature\n\r");
	messageBuf[0] = 0xf4;
	messageBuf[1] = 0x2e;
    TWI_Write(BMP180TWIADDRESS,messageBuf,2);
	delay_ms(7);
	messageBuf[0] = 0xf6;
    TWI_Write(BMP180TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(BMP180TWIADDRESS,2);
	u.tempbuf[0] = messageBuf[1];
	u.tempbuf[1] = messageBuf[0];
	//printf("temp mb[0]= %x ",messageBuf[0]);
	//printf("mb[1]= %x ",messageBuf[1]);
	//printf("%x\n\r",u.outint);
	//printf("BMP Temperature raw in call=%d\n\r",u.outint);

    return u.outint;
}

float CalculateBMP180Temperature(unsigned int BMPtemperature)
{
	volatile long x1,x2,temperatureinC;

	x1 = ((unsigned long)(BMPtemperature - (long)bmpac6) * (long)bmpac5)/(long)32768;
	x2 = ((long)bmpmc*2048)/(x1+(long)bmpmd);
	b5 = x1 + x2;
	temperatureinC = (b5 + 8)/16;
	return ((float)temperatureinC)/10.0;
}

float CalculateBMP180pressurefloat(unsigned int BMPtemperature,unsigned int BMPpressure)
{
	volatile long x1,x2,temperatureinC;
	volatile long b6,x3,b3,p;
	volatile unsigned long b4,b7;
	volatile float compensatedpressure;

	x1 = ((unsigned long)(BMPtemperature - (long)bmpac6) * (long)bmpac5)/(long)32768;
	x2 = ((long)bmpmc*2048)/(x1+(long)bmpmd);
	b5 = x1 + x2;
	temperatureinC = (b5 + 8)/16;
	
	b6 = b5 - 4000;
	x1 = ((long)bmpb2*((b6*b6)/4096))/2048;
	x2 = ((long)bmpac2*b6)/2048;
	x3 = x1 + x2;
	b3 = ((((long)bmpac1*4+x3)<<0)+2)/4;
	x1 = ((long)bmpac3*b6)/8192;
	x2 = ((long)bmpb1*((b6*b6)/4096))/65536;
	x3 = ((x1+x2)+2)/4;
	b4 = ((long)bmpac4*(unsigned long)(x3+32768))/32768;
	b7 = ((unsigned long)BMPpressure-b3)*((unsigned long)50000>>0);
	if(b7<0x80000000) p = (b7*2)/b4;
	else p = (b7/b4)*2;
	x1 = (p/256)*(p/256);
	x1 = (x1*3038)/65536;
	x2 = (-7357*p)/65536;
	p = p + (x1+x2+3791)/16;
	
	compensatedpressure = (float)p*0.00014504;
	return compensatedpressure;
}

//  BMP085 Read Update State Machine
//  The BMP085 takes 5msec to update the temperature ready from
//  a read request and the pressure takes about 28msec but both
//  parameters are read using the same read register so the state
//  machine needs to keep track of which parameter is being accessed
//  and not hold up the access of other mems sensors.
void Check_BMP180_Update(void)
{
	volatile unsigned int BMPtemperature;
	volatile long x1,x2,temperatureinC;
	volatile unsigned int time;
	
	//printf(" in update ");
	// See which parameter is active
	if(bmp180state==BMPTEMP) { // Update temperature
		// If temp reading hasn't started - start it
		if(bmp180tempstate==BMPTEMPIDLE) {
			//printf(" start temp ");
			bmp180tempstate = BMPTEMPREAD;
			bmp180countdown = BMPTEMPDELAY;
			//StartTimermsec(TIMER8MSEC,8);
			Start_Read_BMP180_Temperature();
			//printf("Start read temp\n\r");
			}
		// Else grab the temperature reading and switch sensors
		else if(bmp180countdown==0) {
			//printf(" read temp ");
			BMPtemperature = Finish_Read_BMP180_Temperature();
			ibmptemperature = BMPtemperature;  // used in data logger routine
			x1 = ((long)(BMPtemperature - (long)bmpac6) * (long)bmpac5)/(long)32768;
			x2 = ((long)bmpmc*2048)/(x1+(long)bmpmd);
			b5 = x1 + x2;
			temperatureinC = (b5 + 8)/16;
			bmptemperature = ((float)temperatureinC)/10.0;
			// Number of pressure readings to take before reading temp again
			bmpskipcount = BMPSKIPVALUE;
			bmp180state = BMPPRESSURE;
			bmp180tempstate = BMPTEMPIDLE;
			}
		}
	else {
		if(bmp180pressurestate==BMPPRESSUREIDLE) {
			//printf(" start press ");
			bmp180pressurestate = BMPPRESSUREREAD;
			bmp180countdown = BMPPRESSUREDELAY;
			Start_Read_BMP180_Pressure();
			}
		else {
			if(bmp180countdown==0) {
				//printf(" read press ");
				bmppressure = Finish_Read_BMP180_Pressurefloat();
				if(bmpskipcount--==0) {
					bmp180state = BMPTEMP;
					bmp180pressurestate = BMPPRESSUREIDLE;
					}
				else {
					Start_Read_BMP180_Pressure();
					bmp180countdown = BMPPRESSUREDELAY;
					}
				}
			}
		}
}

// Cycle the BMP180 update state machine to load
// initial temperature and pressure readings
void Init_BMP180_Readings(void)
{
	bmpskipcount = BMPSKIPVALUE;
	bmp180state = BMPTEMP;
	bmp180tempstate = BMPTEMPIDLE;
	bmp180pressurestate = BMPPRESSUREIDLE;
}

void BMP180_PrintData(void)
{
	char outstrg[15];
	
	//printf("\n\r %.2f ct=%.2f %.4f ",bmptemperature,bmptemperature+BMP180TEMPOFFSET,bmppressure);
	//printf("%.1f ",(bmppressure/14.503773773)*1000.0);  // height above sea level in feet - estimated
	//printf("%.1f\n\r-",(14.696-bmppressure)/0.000407);  // height above sea level in meters
}

void DisplayBMP180Brief(char mode)
{
	char outstrg[22];
	volatile long x1,x2,b5,temperatureinC,BMPtemperature;
	float pressure,temperature,presstemp;
	union {
		unsigned char tempbuf[4];
		float pressure;
		} baro;
	union {
		unsigned char tempbuf[4];
		float temperature;
		} temp;
	
	// Get raw temperature and compensate
	BMPtemperature = Read_BMP180_Temperature();
	//printf("temp raw=%d\n\r",BMPtemperature);
	//BMPtemperature = (long)33129;
	x1 = ((long)(BMPtemperature - (long)bmpac6) * (long)bmpac5)/(long)32768;
	x2 = ((long)bmpmc*2048)/(x1+(long)bmpmd);
	b5 = x1 + x2;
	
	if(0) {
/*
		printf("bmpac5=%ld ",bmpac5);
		printf("bmpac6=%ld ",bmpac6);
		printf("bmpmd=%ld ",bmpmd);
		printf("bmpmc=%ld\n\r",bmpmc);
		printf("bmptemp-bmpac6=%ld ",(long)(BMPtemperature - (long)bmpac6));
		printf("(bmptemp-bmpac6)*bmpac5=%ld\n\r",(long)(BMPtemperature - (long)bmpac6) * (long)bmpac5);
		printf("x1=%ld ",x1);
		printf("x2=%ld ",x2);
		printf("b5=%ld\n\r",b5);
*/
		}
	
	temperatureinC = (b5 + 8)/16;
	temperature = temp.temperature = ((float) temperatureinC)/10.0;
	
	pressure = Read_BMP180_pressurefloat(QUIET);
	baro.pressure = pressure;
	
	if(mode=='b') {
		PutChar(temp.tempbuf[0]);
		PutChar(temp.tempbuf[1]);
		PutChar(temp.tempbuf[2]);
		PutChar(temp.tempbuf[3]);
		PutChar(baro.tempbuf[0]);
		PutChar(baro.tempbuf[1]);
		PutChar(baro.tempbuf[2]);
		PutChar(baro.tempbuf[3]);
		}
	else if(mode=='a') {
/*		printf("\n\r %.2f ",temperature);
		printf("ct=%.2f ",temperature+BMP180TEMPOFFSET);
		printf("%.4f ",pressure);
		printf("%.2f ",(pressure/14.503773773)*1000.0);  // height above sea level in feet - estimated
		printf("%.2f\n\r-",(14.696-pressure)/0.000407);  // height above sea level in meters
*/		}
	if((mode=='a')||(mode=='c')) {
#ifdef ST7735LCD
		//strcpy(outstrg,"Temperature=");
		//strcat(outstrg,ftoa(temperature,2,6));
		sprintf(outstrg,"Temperature=%.2f C",temperature);
		ST7735_fillRect(75, 60, 48, 10,colorsRGB24toRGB565(255,255,255));
		ST7735_drawString(1,60,outstrg,colorsRGB24toRGB565(0,0,0),1);
		//strcpy(outstrg,"Pressure=");
		//strcat(outstrg,ftoa(pressure,4,8));
		//strcat(outstrg," psi");
		sprintf(outstrg,"Pressure=%.4f psi",pressure);
		ST7735_fillRect(50, 71, 50, 10,colorsRGB24toRGB565(255,255,255));  // Clear old values
		ST7735_drawString(1,71,outstrg,colorsRGB24toRGB565(0,0,0),1);	
#endif
		}
}

//**********************************************************************
// FXOS8700 6DOF (3-axis accelerometer and 3-axis magnetometer Functions
//**********************************************************************
extern volatile FXOSRawData fxosRawData;

#ifdef FXOS8700SENSOR

unsigned char FXOS8700_Write_Register(I2C_TypeDef* I2Cx,unsigned char address,unsigned char registerid,unsigned char regvalue)
{
	messageBuf[0] = registerid;
	messageBuf[1] = regvalue;
	I2C_WriteBytes(I2Cx,address,2);
	return 0;
}

// Function configures FXOS8700CQ combination accelerometer and magnetometer sensor
int FXOS8700CQ_Init(I2C_TypeDef* I2Cx,unsigned char address)
{
	// Read and check the FXOS8700CQ WHOAMI register
	if(I2C_WriteReadBytes(I2Cx,address,FXOS8700CQ_WHOAMI)!=FXOS8700CQ_WHOAMI_VAL) return I2C_ERROR;

	// write 0000 0000 = 0x00 to accelerometer control register 1 to place FXOS8700CQ into
	// standby
	// [7-1] = 0000 000
	// [0]: active=0
	FXOS8700_Write_Register(I2Cx,address,FXOS8700CQ_CTRL_REG1,0x00);

	// write 0001 1111 = 0x1F to magnetometer control register 1
	// [7]: m_acal=0: auto calibration disabled
	// [6]: m_rst=0: no one-shot magnetic reset
	// [5]: m_ost=0: no one-shot magnetic measurement
	// [4-2]: m_os=111=7: 8x oversampling (for 200Hz) to reduce magnetometer noise
	// [1-0]: m_hms=11=3: select hybrid mode with accel and magnetometer active
	FXOS8700_Write_Register(I2Cx,address,FXOS8700CQ_M_CTRL_REG1,0x1F);

	// write 0010 0000 = 0x20 to magnetometer control register 2
	// [7]: reserved
	// [6]: reserved
	// [5]: hyb_autoinc_mode=1 to map the magnetometer registers to follow the
	// accelerometer registers
	// [4]: m_maxmin_dis=0 to retain default min/max latching even though not used
	// [3]: m_maxmin_dis_ths=0
	// [2]: m_maxmin_rst=0
	// [1-0]: m_rst_cnt=00 to enable magnetic reset each cycle
	FXOS8700_Write_Register(I2Cx,address,FXOS8700CQ_M_CTRL_REG2,0x20);

	// write 0000 0001= 0x01 to XYZ_DATA_CFG register
	// [7]: reserved
	// [6]: reserved
	// [5]: reserved
	// [4]: hpf_out=0
	// [3]: reserved
	// [2]: reserved
	// [1-0]: fs=01 for accelerometer range of +/-4g range with 0.488mg/LSB
	FXOS8700_Write_Register(I2Cx,address,FXOS8700CQ_XYZ_DATA_CFG,0x01);

	// write 0000 1101 = 0x0D to accelerometer control register 1
	// [7-6]: aslp_rate=00
	// [5-3]: dr=001 for 200Hz data rate (when in hybrid mode)
	// [2]: lnoise=1 for low noise mode
	// [1]: f_read=0 for normal 16 bit reads
	// [0]: active=1 to take the part out of standby and enable sampling
	FXOS8700_Write_Register(I2Cx,address,FXOS8700CQ_CTRL_REG1,0x0D);

	// normal return
	return (I2C_OK);
}

// Read status and the three channels of accelerometer and magnetometer data from
// FXOS8700CQ (12 bytes)
unsigned char FXOS8700CQ_Accel_Mag(I2C_TypeDef* I2Cx,unsigned char address)
{
	//uint8_t Buffer[FXOS8700CQ_READ_LEN];
	uint8_t *Buffer;
	int i;
	
	Buffer = messageBuf;

	// Read FXOS8700CQ_LEN=12 bytes (six channels of data)

	//TIM7_Start();
	//for(i=0; i<12; i++) Buffer[i] = I2C_WriteReadBytes(I2Cx,address,FXOS8700CQ_MXOUT_DATA+i,1);
	I2C_WriteReadBytes(I2Cx,address,FXOS8700CQ_MXOUT_DATA,12);
	//TIM7_Stop();
	//printf("Sensor %x read in %ld usec ",address,ElapsedTimeusec());

	// Copy the magnetometer byte data into 16 bit words
	fxosRawData.xm = (Buffer[0] << 8) | Buffer[1];
	fxosRawData.ym = (Buffer[2] << 8) | Buffer[3];
	fxosRawData.zm = (Buffer[4] << 8) | Buffer[5];

	// Copy the 14 bit accelerometer byte data into 16 bit words
	fxosRawData.xa = (int16_t)(((Buffer[6] << 8) | Buffer[7]))<<2;
	fxosRawData.ya = (int16_t)(((Buffer[8] << 8) | Buffer[9]))<<2;
	fxosRawData.za = (int16_t)(((Buffer[10] << 8) | Buffer[11]))<<2;

	fxosRawData.xa /= 4;
	fxosRawData.ya /= 4;
	fxosRawData.za /= 4;

		// Copy the 14 bit accelerometer byte data into 16 bit words
	//fxosRawData.xa = (int16_t)(((Buffer[6] << 8) | Buffer[7]));
	//fxosRawData.ya = (int16_t)(((Buffer[8] << 8) | Buffer[9]));
	//fxosRawData.za = (int16_t)(((Buffer[10] << 8) | Buffer[11]));

	// Normal return
	return (I2C_OK);
}

FXOS_F FXOS8700CQ_Accel_Mag_Float(I2C_TypeDef* I2Cx,unsigned char address)
{
	FXOS_F fdata;
	
	//TIM7_Start();
	FXOS8700CQ_Accel_Mag(I2Cx,address);
	//printf("Sensor %x read in %ld usec ",address,ElapsedTimeusec());

	fdata.mx = (float)fxosRawData.xm/MAGSCALE10;
	fdata.my = (float)fxosRawData.ym/MAGSCALE10;
	fdata.mz = (float)fxosRawData.zm/MAGSCALE10;
	fdata.ax = (float)fxosRawData.xa*ACCELSCALE;
	fdata.ay = (float)fxosRawData.ya*ACCELSCALE;
	fdata.az = (float)fxosRawData.za*ACCELSCALE;
	
	// Transfer the raw data to the appropriate finger structures
	//  TBD - redo these functions so data isn't copied like this
	fdata.mxi = fxosRawData.xm;
	fdata.myi = fxosRawData.ym;
	fdata.mzi = fxosRawData.zm;
	fdata.axi = fxosRawData.xa;
	fdata.ayi = fxosRawData.ya;
	fdata.azi = fxosRawData.za;


	return fdata;
}

void FXOS8700_Init_Chain(I2C_TypeDef* I2Cx)
{
	FXOS8700CQ_Init(I2Cx,0x1C);
	FXOS8700CQ_Init(I2Cx,0x1D);
	FXOS8700CQ_Init(I2Cx,0x1F);
}

void FXOS8700_Find(I2C_TypeDef* I2Cx)
{
	if(I2C_WriteReadBytes(I2Cx,0x1C,FXOS8700CQ_WHOAMI,1)==FXOS8700CQ_WHOAMI_VAL)
		printf("FXOS8700 found at 0x1C\n\r");
	else printf("FXOS8700 not found at 0x1C\n\r");
	
	//delay_ms(2);
	
	if(I2C_WriteReadBytes(I2Cx,0x1D,FXOS8700CQ_WHOAMI,1)==FXOS8700CQ_WHOAMI_VAL)
		printf("FXOS8700 found at 0x1D\n\r");
	else printf("FXOS8700 not found at 0x1D\n\r");
	
	//delay_ms(2);

	if(I2C_WriteReadBytes(I2Cx,0x1F,FXOS8700CQ_WHOAMI,1)==FXOS8700CQ_WHOAMI_VAL)
		printf("FXOS8700 found at 0x1F\n\r");
	else printf("FXOS8700 not found at 0x1F\n\r");

	//delay_ms(2);
}
#endif

// PCA9540 is a two channel I2C channel selector
void PCA9540_ChannelSelect(unsigned char channel)
{
	messageBuf[0] = channel;
	TWI_Write(PCA9540_SLAVE_ADDRESS,messageBuf,1);
}


//**********************************************************************
// TWI Enumeration Functions
//**********************************************************************
void Enumerate_BMP180(void)
{
	BMPstate = bmpac1 = 0;

	bmpac1 = Read_BMP180_EEPROM_Signed(0xaa);
	//printf("bmpac1=%x\n\r");
	if(bmpac1!=0) {
		BMPstate |= 0x01;
		delay_ms(7);
	}
}

#define MPU9150_WHO_AM_I           0x75   // R


void Enumerate_MPU9150(void)
{
	unsigned char whoami;
	mpu9150state = 0;

	whoami = MPU9150_ReadSensor2(MPU9150_WHO_AM_I);
	//printf("whoami=%x\n\r",whoami);
	if(whoami==0x68) {
		mpu9150state |= 0x01;
		delay_ms(7);
	}
}

#ifdef FXOS8700SENSOR
void Enumerate_FXOS8700(void)
{
	unsigned char whoami;
	fxos8700state = 0;

	if(I2C_WriteReadBytes(I2C2,0x1C,FXOS8700CQ_WHOAMI,1)==FXOS8700CQ_WHOAMI_VAL) {
		fxos8700state |= 0x01;
		delay_ms(7);
		}	
}
#endif

#ifdef FSI7013SENSOR
unsigned char SI7013_Read_Register(unsigned char address,unsigned char reg)
{
	messageBuf[0] = reg;
	TWI_Write(address,messageBuf,1);
	messageBuf[0] = 0;
	TWI_Start_Read(address,1);
	return messageBuf[0];
}

volatile unsigned int SI7013_Read_Word(unsigned char address)
{
	status_code_t twistatus;
	
	union {
		unsigned int intvalue;
		unsigned char byte[2];
	} si;
	
	messageBuf[0] = messageBuf[1] = messageBuf[3] = 0;
	twistatus = TWI_Start_Read(address,2);
	si.byte[1] = messageBuf[0];
	si.byte[0] = messageBuf[1];
	
	return si.intvalue;
}

void SI7013_Read_Register_Numbytes(unsigned char address,unsigned char reg,int numbytes)
{
	messageBuf[0] = reg;
	TWI_Write(address,messageBuf,1);
	messageBuf[0] = 0;
	TWI_Start_Read(address,numbytes);
}

void SI7013_Read_Numbytes(unsigned char address,int numbytes)
{
	TWI_Start_Read(address,numbytes);
}

unsigned int SI7013_Read_Register_Word(unsigned char address,unsigned char reg)
{
	union {
		unsigned int intvalue;
		unsigned char byte[2];
		} si;
		
	messageBuf[0] = reg;
	TWI_Write(address,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(address,2);
	si.byte[1] = messageBuf[0];
	si.byte[0] = messageBuf[1];
	return si.intvalue;
}

void SI7013_Write_Register(unsigned char address,unsigned char reg)
{
	messageBuf[0] = reg;
	TWI_Write(address,messageBuf,1);
}

void SI7013_Write_Register_Value(unsigned char address,unsigned char reg,unsigned char regvalue)
{
	messageBuf[0] = reg;
	messageBuf[1] = regvalue;
	TWI_Write(address,messageBuf,2);
}

void Enumerate_SI7013(void)
{
	unsigned char whoami;
	si7013state = 0;

	if(SI7013_Read_Register(SI7013_SLAVE_ADDRESS,SI7013_READ_REG1)==0x3A) {
		si7013state |= 0x01;
		Delaymsec(7);
	}
}

volatile void SI7013_ReadHumidity(void)
{
	si7013.rawhumidity = SI7013_Read_Word(SI7013_SLAVE_ADDRESS);
	si7013.humidity = ((125.0 * (float)si7013.rawhumidity)/65536.0) - 6.0;
}

volatile void SI7013_ReadTemp(void)
{
	si7013.rawtemperature = SI7013_Read_Word(SI7013_SLAVE_ADDRESS);
	si7013.temperature = (157.72*(float)si7013.rawtemperature/65536.0) - 46.85;
}

void SI7013_ReadTempHumidity(void)
{
	SI7013_Write_Register(SI7013_SLAVE_ADDRESS,SI7013_RH_NOHOLD);  // 250usecond
	delay_ms(13);
	SI7013_ReadHumidity();
	SI7013_Write_Register(SI7013_SLAVE_ADDRESS,SI7013_TEMP_NOHOLD);
	delay_ms(13);
	SI7013_ReadTemp();
}

void SI7013_Test(void)
{
	char inchar;
	int i;
	
	printf("\n\rSI7013 Humidity Sensor data\n\r");
	printf("Enter command:  r,h,s\n\r>");
	
	while((inchar=getnwchar())!='e') {
		switch (inchar) {
			case 'f':
				SI7013_Write_Register_Value(SI7013_SLAVE_ADDRESS,SI7013_FIRM_REV1,SI7013_FIRM_REV2);
				SI7013_Read_Numbytes(SI7013_SLAVE_ADDRESS,1);
				printf("r\n\rSI7013 Firmware version=%x\n\r",messageBuf[0]);
				break;
			case 'r':
				printf("r\n\rSI7013_READ_REG1=%x\n\r",SI7013_Read_Register(SI7013_SLAVE_ADDRESS,SI7013_READ_REG1));
				printf("SI7013_READ_REG2=%x\n\r",SI7013_Read_Register(SI7013_SLAVE_ADDRESS,SI7013_READ_REG2));
				printf("SI7013_READ_REG3=%x\n\r>",SI7013_Read_Register(SI7013_SLAVE_ADDRESS,SI7013_READ_REG3));
				break;
			case 'h':
				SI7013_ReadTempHumidity();
				printf("h\n\rSI7013 Humidity =%s",ftoa(si7013.humidity,2,7));
				printf("  t=%s\n\r>",ftoa(si7013.temperature,2,7));
				break;
			case 's':
				SI7013_Write_Register_Value(SI7013_SLAVE_ADDRESS,SI7013_READ_ID_1_1,SI7013_READ_ID_1_2);
				//SI7013_Write_Register_Value(SI7013_SLAVE_ADDRESS,SI7013_SERIAL_1,SI7013_SERIAL_1_CRC);
				delay_ms(15);
				SI7013_Read_Numbytes(SI7013_SLAVE_ADDRESS,8);
				printf("s\n\rFirst serial: ");
				for(i=0; i<8; i++) printf("%x ",messageBuf[i]);
				printf("\n\r");
				SI7013_Write_Register_Value(SI7013_SLAVE_ADDRESS,SI7013_READ_ID_2_1,SI7013_READ_ID_2_2);
				//SI7013_Write_Register_Value(SI7013_SLAVE_ADDRESS,SI7013_SERIAL_2,SI7013_SERIAL_2_CRC);
				delay_ms(15);
				SI7013_Read_Numbytes(SI7013_SLAVE_ADDRESS,6);
				printf("Second serial: ");
				for(i=0; i<6; i++) printf("%x ",messageBuf[i]);
				printf("\n\r>");
				break;
			case CR:
				printf("\n\r>");
				break;
			}
		}
}
#endif

void Enumerate_TWI_Devices(char mode)
{
	if(mode==VERBOSE) Puts("\n\rEnumerate TWI Devices\n\r");

	// Enumerate MPU9150
	Enumerate_MPU9150();
	if(mode==VERBOSE) {
		if(mpu9150state==0) Puts(" MPU9150 not found,");
		else Puts(" MPU9150 found,");
		}

#ifdef FXOS8700SENSOR
	// Enumerate FXOS8700
	// TBD - find all sensors if PicoM305
	Enumerate_FXOS8700();
	if(mode==VERBOSE) {
		if(fxos8700state==0) printf(" FXOS8700 not found");
		else printf(" FXOS8700 found");
		}
#endif
#ifdef SI7013SENSOR
// Enumerate SI7013 Humidity sensor
	Enumerate_SI7013();
	if(mode==VERBOSE) {
		if(si7013state==0) printf(" SI7013 not found");
		else printf(" SI7013 found");
		}
#endif

	// Enumerate SmartLCD
	//Enumerate_SmartLCD();
	//if(smartlcdstate==0) putcs(" SmartLCD not found,");
	//else putcs(" SmartLCD found,");
}

// Update the mems sensors - this function is called by the top level
// command processor and runs continuously if enabled
void MEMS_Sensor_Update(void)
{
	if(mpu9150state) MPU9150_Data_Read();

	// TBD - sequence reading of BMP180 data
	//if(BMPstate) DisplayBMP180Brief('n');
	if(BMPstate) Check_BMP180_Update(); 
	
	if(l3gd20state) ;
	
	if(fxos8700state) ;
	
	if(mma8653state) ;
	

#ifdef FSI7013SENSOR
	// SI7013 sensor takes 13msecond to produce a new reading after a
	// No hold humidity read request is started.  Humidity is read first
	// then temperature with a 13 msec delay between start and finish of read
	if(si7013state) {
		if(si7013sample==SI7013START) {
			SI7013_Write_Register(SI7013_SLAVE_ADDRESS,SI7013_RH_NOHOLD);
			//else SI7013_Write_Register(SI7013_SLAVE_ADDRESS,SI7013_TEMP_NOHOLD);
			mseccountdown2 = 25;
			si7013sample = SI7013WAITING;
			//printf("Sample %c at %ld ",si7013sampletarget,mseccounter);
			}
		else if(!mseccountdown2) {
			if(si7013sample==SI7013DELAYING) si7013sample = SI7013START;
			else {
				SI7013_ReadHumidity();
				SI7013_Write_Register(SI7013_SLAVE_ADDRESS,SI7013_TEMP_PRE_RH);
				SI7013_ReadTemp();
				mseccountdown2 = SI7013SAMPLERATE - 25;
				si7013sample = SI7013DELAYING;
				}
			}
		}
#endif

	if(adps9007state) ;
}

#ifdef FXOS8700SENSOR
void FXOS8700_Init_All(char mode)
{
	int i;

	if(mode==VERBOSE) printf("\n\rFind all 15 FXOS8700's on all five chains\n\r");
	// Find and initialize all 15 finger sensors
	for(i=0; i<4; i++) {
		if(mode==VERBOSE) printf("\n\rSearch Femto Chain %d %x\n\r",i,pca9545channel[i]);
		messageBuf[0] = pca9545channel[i];
		I2C_WriteBytes(I2C1,PCA9545_SLAVE_ADDRESS,1);
		FXOS8700_Find(I2C1);
		FXOS8700_Init_Chain(I2C1);
		}
	if(mode==VERBOSE) printf("\n\rSearch Femto Chain 4\n\r");
	FXOS8700_Find(I2C2);
	FXOS8700_Init_Chain(I2C2);
}

void FXOS8700_Display_All(char mode)
{
	int i;

	//if(mode==VERBOSE) printf("\n\rDisplay all 15 sensor data\n\r");
	if(mode==VERBOSE) printf("\n\r");
	for(i=0; i<4; i++) {
		messageBuf[0] = pca9545channel[i];
		I2C_WriteBytes(I2C1,PCA9545_SLAVE_ADDRESS,1);
		mcp[i] = FXOS8700CQ_Accel_Mag_Float(I2C1,FXOS8700CQ_MCP_SLAVE_ADDR);
		//printf("\n\r");
		pip[i] = FXOS8700CQ_Accel_Mag_Float(I2C1,FXOS8700CQ_PIP_SLAVE_ADDR);
		//printf("\n\r");
		dip[i] = FXOS8700CQ_Accel_Mag_Float(I2C1,FXOS8700CQ_DIP_SLAVE_ADDR);
		//printf("\n\r");
		if(mode==VERBOSE) printf("f%d: d=%6.3f %6.3f %6.3f   p=%6.3f %6.3f %6.3f   m=%6.3f %6.3f %6.3f \n\r",i,
						dip[i].ax,dip[i].ay,dip[i].az,pip[i].ax,pip[i].ay,pip[i].az,mcp[i].ax,mcp[i].ay,mcp[i].az);
		}

	mcp[4] = FXOS8700CQ_Accel_Mag_Float(I2C2,FXOS8700CQ_MCP_SLAVE_ADDR);
	//printf("\n\r");
	pip[4] = FXOS8700CQ_Accel_Mag_Float(I2C2,FXOS8700CQ_PIP_SLAVE_ADDR);
	//printf("\n\r");
	dip[4] = FXOS8700CQ_Accel_Mag_Float(I2C2,FXOS8700CQ_DIP_SLAVE_ADDR);
	if(mode==VERBOSE) printf("f%d: d=%6.3f %6.3f %6.3f   p=%6.3f %6.3f %6.3f   m=%6.3f %6.3f %6.3f",4,
					dip[4].ax,dip[4].ay,dip[4].az,pip[4].ax,pip[4].ay,pip[4].az,mcp[4].ax,mcp[4].ay,mcp[4].az);
}
#endif


/***************************************************************************************
 SSD1306 I2C Control Functions

 This ssd1306 set of functions is drawn from three sources:
 some code written by soonc found here ->  http://www.ccsinfo.com/forum/viewtopic.php?t=52836
 Bulk of code written by kmm found here -> https://github.com/kmm/SS1306
 Line/circle routines written by Shawon Shahryiar   https://www.youtube.com/watch?v=qdZXUppUpIQ

 The three code bases were combined by SS to create this version.  The first two don't rely on
 a buffer while the last one does.  The buffered and unbuffered version have not bee integrated so
 mixing text and lines is not possible yet.
*******************************************************************************************/

#define SSD1306_I2C_ADDRESS  0x3C
//#define SSD1306_I2C_ADDRESS  0x3D

#define SSD1306_LCDWIDTH      128
#define SSD1306_LCDHEIGHT      64
#define SSD1306_SETCONTRAST   0x81
#define SSD1306_DISPLAYALLON_RESUME 0xA4
#define SSD1306_DISPLAYALLON 0xA5
#define SSD1306_NORMALDISPLAY 0xA6
#define SSD1306_INVERTDISPLAY 0xA7
#define SSD1306_DISPLAYOFF 0xAE
#define SSD1306_DISPLAYON 0xAF
#define SSD1306_SETDISPLAYOFFSET 0xD3
#define SSD1306_SETCOMPINS 0xDA
#define SSD1306_SETVCOMDETECT 0xDB
#define SSD1306_SETDISPLAYCLOCKDIV 0xD5
#define SSD1306_SETPRECHARGE 0xD9
#define SSD1306_SETMULTIPLEX 0xA8
#define SSD1306_SETLOWCOLUMN 0x00
#define SSD1306_SETHIGHCOLUMN 0x10
#define SSD1306_SETSTARTLINE 0x40
#define SSD1306_MEMORYMODE 0x20
#define SSD1306_COLUMNADDR 0x21
#define SSD1306_PAGEADDR   0x22
#define SSD1306_COMSCANINC 0xC0
#define SSD1306_COMSCANDEC 0xC8
#define SSD1306_SEGREMAP 0xA0
#define SSD1306_CHARGEPUMP 0x8D
#define SSD1306_EXTERNALVCC 0x1
#define SSD1306_SWITCHCAPVCC 0x2

#define BOX_TL 0x10
#define BOX_TR 0x0E
#define BOX_HL 0x40
#define BOX_VL 0x5D
#define BOX_BL 0x0D
#define BOX_BR 0x1D

#define BOX_TL_ERASE 0xEF
#define BOX_TR_ERASE 0xF1
#define BOX_HL_ERASE 0xBF
#define BOX_VL_ERASE 0x92
#define BOX_BL_ERASE 0xF2
#define BOX_BR_ERASE 0xE2

#define SS1306_OLED_GEOM_W 128
#define SS1306_OLED_GEOM_H 64

/*****************************************************************
Character generator table is stored in progmem as "font", and contains
128 cells in PETSCII layout (i.e. you can generate them from C64 font files).
Included is the canonical 8x8 C64 font, with a couple minor changes to make
box-drawn digits look better. Characters are rotated 90 degrees clockwise
(so we don't have to waste AVR cycles flipping tiles). The pointy part of
an 'A' should point that way -> if you're doing it right.

A character generation utility is here: http://www.min.at/prinz/o/software/pixelfont/

This font table generates incorrect a and z fonts - ss
***********************************************************************/
//const unsigned char font[1024] PROGMEM =  // version for AVR's
const unsigned char font[1024]  =           // version for AVR32
{
	0x00, 0x3E, 0x7F, 0x41, 0x4D, 0x4F, 0x2E, 0x00,	// Char 000 (.)
	0x00, 0x20, 0x74, 0x54, 0x54, 0x7C, 0x78, 0x00,	// Char 001 (.)
	0x00, 0x7E, 0x7E, 0x48, 0x48, 0x78, 0x30, 0x00,	// Char 002 (.)
	0x00, 0x38, 0x7C, 0x44, 0x44, 0x44, 0x00, 0x00,	// Char 003 (.)
	0x00, 0x30, 0x78, 0x48, 0x48, 0x7E, 0x7E, 0x00,	// Char 004 (.)
	0x00, 0x38, 0x7C, 0x54, 0x54, 0x5C, 0x18, 0x00,	// Char 005 (.)
	0x00, 0x00, 0x08, 0x7C, 0x7E, 0x0A, 0x0A, 0x00,	// Char 006 (.)
	0x00, 0x98, 0xBC, 0xA4, 0xA4, 0xFC, 0x7C, 0x00,	// Char 007 (.)
	0x00, 0x7E, 0x7E, 0x08, 0x08, 0x78, 0x70, 0x00,	// Char 008 (.)
	0x00, 0x00, 0x48, 0x7A, 0x7A, 0x40, 0x00, 0x00,	// Char 009 (.)
	0x00, 0x00, 0x80, 0x80, 0x80, 0xFA, 0x7A, 0x00,	// Char 010 (.)
	0x00, 0x7E, 0x7E, 0x10, 0x38, 0x68, 0x40, 0x00,	// Char 011 (.)
	0x00, 0x00, 0x42, 0x7E, 0x7E, 0x40, 0x00, 0x00,	// Char 012 (.)
	0x00, 0x7C, 0x7C, 0x18, 0x38, 0x1C, 0x7C, 0x78,	// Char 013 (.)
	0x00, 0x7C, 0x7C, 0x04, 0x04, 0x7C, 0x78, 0x00,	// Char 014 (.)
	0x00, 0x38, 0x7C, 0x44, 0x44, 0x7C, 0x38, 0x00,	// Char 015 (.)
	0x00, 0xFC, 0xFC, 0x24, 0x24, 0x3C, 0x18, 0x00,	// Char 016 (.)
	0x00, 0x18, 0x3C, 0x24, 0x24, 0xFC, 0xFC, 0x00,	// Char 017 (.)
	0x00, 0x7C, 0x7C, 0x04, 0x04, 0x0C, 0x08, 0x00,	// Char 018 (.)
	0x00, 0x48, 0x5C, 0x54, 0x54, 0x74, 0x24, 0x00,	// Char 019 (.)
	0x00, 0x04, 0x04, 0x3E, 0x7E, 0x44, 0x44, 0x00,	// Char 020 (.)
	0x00, 0x3C, 0x7C, 0x40, 0x40, 0x7C, 0x7C, 0x00,	// Char 021 (.)
	0x00, 0x1C, 0x3C, 0x60, 0x60, 0x3C, 0x1C, 0x00,	// Char 022 (.)
	0x00, 0x1C, 0x7C, 0x70, 0x38, 0x70, 0x7C, 0x1C,	// Char 023 (.)
	0x00, 0x44, 0x6C, 0x38, 0x38, 0x6C, 0x44, 0x00,	// Char 024 (.)
	0x00, 0x9C, 0xBC, 0xA0, 0xE0, 0x7C, 0x3C, 0x00,	// Char 025 (.)
	0x00, 0x44, 0x64, 0x74, 0x5C, 0x4C, 0x44, 0x00,	// Char 026 (.)
	0x00, 0x00, 0x7F, 0x7F, 0x41, 0x41, 0x00, 0x00,	// Char 027 (.)
	0x40, 0x68, 0x7C, 0x5E, 0x49, 0x49, 0x22, 0x00,	// Char 028 (.)
	0x00, 0x00, 0x41, 0x41, 0x7F, 0x7F, 0x00, 0x00,	// Char 029 (.)
	0x00, 0x08, 0x0C, 0xFE, 0xFE, 0x0C, 0x08, 0x00,	// Char 030 (.)
	0x00, 0x18, 0x3C, 0x7E, 0x18, 0x18, 0x18, 0x18,	// Char 031 (.)
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Char 032 ( )
	0x00, 0x00, 0x00, 0x4F, 0x4F, 0x00, 0x00, 0x00,	// Char 033 (!)
	0x00, 0x07, 0x07, 0x00, 0x00, 0x07, 0x07, 0x00,	// Char 034 (")
	0x14, 0x7F, 0x7F, 0x14, 0x14, 0x7F, 0x7F, 0x14,	// Char 035 (#)
	0x00, 0x24, 0x2E, 0x6B, 0x6B, 0x3A, 0x12, 0x00,	// Char 036 ($)
	0x00, 0x63, 0x33, 0x18, 0x0C, 0x66, 0x63, 0x00,	// Char 037 (%)
	0x00, 0x32, 0x7F, 0x4D, 0x4D, 0x77, 0x72, 0x50,	// Char 038 (&)
	0x00, 0x00, 0x00, 0x04, 0x06, 0x03, 0x01, 0x00,	// Char 039 (')
	0x00, 0x00, 0x1C, 0x3E, 0x63, 0x41, 0x00, 0x00,	// Char 040 (()
	0x00, 0x00, 0x41, 0x63, 0x3E, 0x1C, 0x00, 0x00,	// Char 041 ())
	0x08, 0x2A, 0x3E, 0x1C, 0x1C, 0x3E, 0x2A, 0x08,	// Char 042 (*)
	0x00, 0x08, 0x08, 0x3E, 0x3E, 0x08, 0x08, 0x00,	// Char 043 (+)
	0x00, 0x00, 0x80, 0xE0, 0x60, 0x00, 0x00, 0x00,	// Char 044 (,)
	0x00, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x00,	// Char 045 (-)
	0x00, 0x00, 0x00, 0x60, 0x60, 0x00, 0x00, 0x00,	// Char 046 (.)
	0x00, 0x40, 0x60, 0x30, 0x18, 0x0C, 0x06, 0x02,	// Char 047 (/)
	0x00, 0x3E, 0x7F, 0x49, 0x45, 0x7F, 0x3E, 0x00,	// Char 048 (0)
	0x00, 0x40, 0x44, 0x7F, 0x7F, 0x40, 0x40, 0x00,	// Char 049 (1)
	0x00, 0x62, 0x73, 0x51, 0x49, 0x4F, 0x46, 0x00,	// Char 050 (2)
	0x00, 0x22, 0x63, 0x49, 0x49, 0x7F, 0x36, 0x00,	// Char 051 (3)
	0x00, 0x18, 0x18, 0x14, 0x16, 0x7F, 0x7F, 0x10,	// Char 052 (4)
	0x00, 0x27, 0x67, 0x45, 0x45, 0x7D, 0x39, 0x00,	// Char 053 (5)
	0x00, 0x3E, 0x7F, 0x49, 0x49, 0x7B, 0x32, 0x00,	// Char 054 (6)
	0x00, 0x03, 0x03, 0x79, 0x7D, 0x07, 0x03, 0x00,	// Char 055 (7)
	0x00, 0x36, 0x7F, 0x49, 0x49, 0x7F, 0x36, 0x00,	// Char 056 (8)
	0x00, 0x26, 0x6F, 0x49, 0x49, 0x7F, 0x3E, 0x00,	// Char 057 (9)
	0x00, 0x00, 0x00, 0x24, 0x24, 0x00, 0x00, 0x00,	// Char 058 (:)
	0x00, 0x00, 0x80, 0xE4, 0x64, 0x00, 0x00, 0x00,	// Char 059 (;)
	0x00, 0x08, 0x1C, 0x36, 0x63, 0x41, 0x41, 0x00,	// Char 060 (<)
	0x00, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x00,	// Char 061 (=)
	0x00, 0x41, 0x41, 0x63, 0x36, 0x1C, 0x08, 0x00,	// Char 062 (>)
	0x00, 0x02, 0x03, 0x51, 0x59, 0x0F, 0x06, 0x00,	// Char 063 (?)
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18,	// Char 064 (@)
	0x00, 0x7C, 0x7E, 0x0B, 0x0B, 0x7E, 0x7C, 0x00,	// Char 065 (A)
	0x00, 0x7F, 0x7F, 0x49, 0x49, 0x7F, 0x36, 0x00,	// Char 066 (B)
	0x00, 0x3E, 0x7F, 0x41, 0x41, 0x63, 0x22, 0x00,	// Char 067 (C)
	0x00, 0x7F, 0x7F, 0x41, 0x63, 0x3E, 0x1C, 0x00,	// Char 068 (D)
	0x00, 0x7F, 0x7F, 0x49, 0x49, 0x41, 0x41, 0x00,	// Char 069 (E)
	0x00, 0x7F, 0x7F, 0x09, 0x09, 0x01, 0x01, 0x00,	// Char 070 (F)
	0x00, 0x3E, 0x7F, 0x41, 0x49, 0x7B, 0x3A, 0x00,	// Char 071 (G)
	0x00, 0x7F, 0x7F, 0x08, 0x08, 0x7F, 0x7F, 0x00,	// Char 072 (H)
	0x00, 0x00, 0x41, 0x7F, 0x7F, 0x41, 0x00, 0x00,	// Char 073 (I)
	0x00, 0x20, 0x60, 0x41, 0x7F, 0x3F, 0x01, 0x00,	// Char 074 (J)
	0x00, 0x7F, 0x7F, 0x1C, 0x36, 0x63, 0x41, 0x00,	// Char 075 (K)
	0x00, 0x7F, 0x7F, 0x40, 0x40, 0x40, 0x40, 0x00,	// Char 076 (L)
	0x00, 0x7F, 0x7F, 0x06, 0x0C, 0x06, 0x7F, 0x7F,	// Char 077 (M)
	0x00, 0x7F, 0x7F, 0x0E, 0x1C, 0x7F, 0x7F, 0x00,	// Char 078 (N)
	0x00, 0x3E, 0x7F, 0x41, 0x41, 0x7F, 0x3E, 0x00,	// Char 079 (O)
	0x00, 0x7F, 0x7F, 0x09, 0x09, 0x0F, 0x06, 0x00,	// Char 080 (P)
	0x00, 0x1E, 0x3F, 0x21, 0x61, 0x7F, 0x5E, 0x00,	// Char 081 (Q)
	0x00, 0x7F, 0x7F, 0x19, 0x39, 0x6F, 0x46, 0x00,	// Char 082 (R)
	0x00, 0x26, 0x6F, 0x49, 0x49, 0x7B, 0x32, 0x00,	// Char 083 (S)
	0x00, 0x01, 0x01, 0x7F, 0x7F, 0x01, 0x01, 0x00,	// Char 084 (T)
	0x00, 0x3F, 0x7F, 0x40, 0x40, 0x7F, 0x3F, 0x00,	// Char 085 (U)
	0x00, 0x1F, 0x3F, 0x60, 0x60, 0x3F, 0x1F, 0x00,	// Char 086 (V)
	0x00, 0x7F, 0x7F, 0x30, 0x18, 0x30, 0x7F, 0x7F,	// Char 087 (W)
	0x00, 0x63, 0x77, 0x1C, 0x1C, 0x77, 0x63, 0x00,	// Char 088 (X)
	0x00, 0x07, 0x0F, 0x78, 0x78, 0x0F, 0x07, 0x00,	// Char 089 (Y)
	0x00, 0x61, 0x71, 0x59, 0x4D, 0x47, 0x43, 0x00,	// Char 090 (Z)
	0x18, 0x18, 0x18, 0xFF, 0xFF, 0x18, 0x18, 0x18,	// Char 091 ([)
	0x33, 0x33, 0xCC, 0xCC, 0x00, 0x00, 0x00, 0x00,	// Char 092 (\)
	0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00,	// Char 093 (])
	0x00, 0x00, 0x00, 0x0F, 0x0F, 0x00, 0x00, 0x00,	// Char 094 (^)
	0x00, 0x00, 0x00, 0xF0, 0xF0, 0x00, 0x00, 0x00,	// Char 095 (_)
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,	// Char 096 (`)
	0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00,	// Char 097 (a)
	0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0,	// Char 098 (b)
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,	// Char 099 (c)
	0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,	// Char 100 (d)
	0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Char 101 (e)
	0x33, 0x33, 0xCC, 0xCC, 0x33, 0x33, 0xCC, 0xCC,	// Char 102 (f)
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF,	// Char 103 (g)
	0x30, 0x30, 0xC0, 0xC0, 0x30, 0x30, 0xC0, 0xC0,	// Char 104 (h)
	0x33, 0x99, 0xCC, 0x66, 0x33, 0x99, 0xCC, 0x66,	// Char 105 (i)
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF,	// Char 106 (j)
	0x00, 0x00, 0x00, 0xFF, 0xFF, 0x18, 0x18, 0x18,	// Char 107 (k)
	0x00, 0x00, 0x00, 0x00, 0xF0, 0xF0, 0xF0, 0xF0,	// Char 108 (l)
	0x00, 0x00, 0x00, 0x1F, 0x1F, 0x18, 0x18, 0x18,	// Char 109 (m)
	0x18, 0x18, 0x18, 0xF8, 0xF8, 0x00, 0x00, 0x00,	// Char 110 (n)
	0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0,	// Char 111 (o)
	0x00, 0x00, 0x00, 0xF8, 0xF8, 0x18, 0x18, 0x18,	// Char 112 (p)
	0x18, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x18,	// Char 113 (q)
	0x18, 0x18, 0x18, 0xF8, 0xF8, 0x18, 0x18, 0x18,	// Char 114 (r)
	0x18, 0x18, 0x18, 0xFF, 0xFF, 0x00, 0x00, 0x00,	// Char 115 (s)
	0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Char 116 (t)
	0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00,	// Char 117 (u)
	0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF,	// Char 118 (v)
	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03,	// Char 119 (w)
	0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07,	// Char 120 (x)
	0xE0, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0,	// Char 121 (y)
	0x00, 0x78, 0x78, 0x30, 0x18, 0x0C, 0x06, 0x03,	// Char 122 (z)
	0xF0, 0xF0, 0xF0, 0xF0, 0x00, 0x00, 0x00, 0x00,	// Char 123 ({)
	0x00, 0x00, 0x00, 0x00, 0x0F, 0x0F, 0x0F, 0x0F,	// Char 124 (|)
	0x18, 0x18, 0x18, 0x1F, 0x1F, 0x00, 0x00, 0x00,	// Char 125 (})
	0x0F, 0x0F, 0x0F, 0x0F, 0x00, 0x00, 0x00, 0x00,	// Char 126 (~)
	0x0F, 0x0F, 0x0F, 0x0F, 0xF0, 0xF0, 0xF0, 0xF0
};

unsigned char pixelbuffer[1056];

//unsigned char display_buffer[1025];
void  ssd1306_command(unsigned char c)
{
	messageBuf[0] = 0x80; // This byte is Command
	messageBuf[1] = c;
	I2C_WriteBytes(I2C1,SSD1306_I2C_ADDRESS,2);
}

void  ssd1306_data(unsigned char c)
{
	messageBuf[0] = 0x40; // This byte is DATA
	messageBuf[1] = c;
	I2C_WriteBytes(I2C1,SSD1306_I2C_ADDRESS,2);
}

void ssd1306_clear_buffer(void)
{
	int i;

    for(i=0; i<1056; i++) pixelbuffer[i] = 0x00;
}

// Init according to SSD1306 data sheet and many places on the web
// Assumes OLED is connected to I2C1
void ssd1306_Init(void)
{
	int i;

	// Init sequence for 128x64 OLED module
	ssd1306_command(SSD1306_DISPLAYOFF);                    // 0xAE
	ssd1306_command(SSD1306_SETDISPLAYCLOCKDIV);            // 0xD5
	ssd1306_command(0x80);                 // the suggested ratio 0x80
	ssd1306_command(SSD1306_SETMULTIPLEX);                  // 0xA8
	ssd1306_command(0x3F);
	ssd1306_command(SSD1306_SETDISPLAYOFFSET);              // 0xD3
	ssd1306_command(0x0);                                   // no offset
	ssd1306_command(SSD1306_SETSTARTLINE);// | 0x0);        // line #0
	ssd1306_command(SSD1306_CHARGEPUMP);                    // 0x8D
	ssd1306_command(0x14);  // using internal VCC
	ssd1306_command(SSD1306_MEMORYMODE);                    // 0x20
	ssd1306_command(0x00);          // 0x00 horizontal addressing
	ssd1306_command(SSD1306_SEGREMAP | 0x1); // rotate screen 180
	ssd1306_command(SSD1306_COMSCANDEC); // rotate screen 180
	ssd1306_command(SSD1306_SETCOMPINS);                    // 0xDA
	ssd1306_command(0x12);
	ssd1306_command(SSD1306_SETCONTRAST);                   // 0x81
	ssd1306_command(0x7f);  // was 0xCF
	ssd1306_command(SSD1306_SETPRECHARGE);                  // 0xd9
	ssd1306_command(0x11);  // was F1
	ssd1306_command(SSD1306_SETVCOMDETECT);                 // 0xDB
	ssd1306_command(0x20);  // was 0x40
	ssd1306_command(SSD1306_NORMALDISPLAY);                 // 0xA6
	ssd1306_command(SSD1306_DISPLAYALLON_RESUME);           // 0xA4
	ssd1306_command(SSD1306_DISPLAYON);                     //switch on OLED

	ssd1306_clear_buffer();
}

void ssd1306_set_clock(uint8_t val)
{
	ssd1306_command(0xD5);
	ssd1306_command(val);
}

void ssd1306_set_addr_mode(uint8_t mode)
{
	// 0 -> horizontal (write column, increment column pointer, at column overrun reset column pointer and increment page pointer)
	// 1 -> vertical (write column, increment page pointer, at page overrun reset page pointer and increment column pointer)
	// 2 -> page (write column, increment column pointer, reset column pointer at overrun)
	ssd1306_command(0x20);
	ssd1306_command((mode < 2) ? mode : 0);
}

void ssd1306_home(void)
{
	ssd1306_command(0x21); // column range
	ssd1306_command(0x00); //  set start to 0
	ssd1306_command(0x7F); //  set end to 0x7F
	ssd1306_command(0x22); // row range
	ssd1306_command(0x00); //  set start to 0
	ssd1306_command(0x07); //  set end to 0x07
}

void ssd1306_move_raw(uint8_t row, uint8_t col)
{
	if(col > 127) { col = 127; }
	if(row > 7) { row = 7; }

	ssd1306_command(0x21);     // set column
	ssd1306_command(col);      // start = col
	ssd1306_command(0x7F);     // end = col max
	ssd1306_command(0x22);     // set row
	ssd1306_command(row);      // start = row
	ssd1306_command(0x07);     // end = row max
}

// Transfers the local buffer to the CGRAM in the SSD1306
/*
void TransferBuffer(void)
{
	// set the Column and Page addresses to 0,0
	ssd1306_home();
	//setPageAddress();

	display_buffer[0] = 0X40; // data not command

	twim_write(BMP085_TWI, display_buffer,1025, SSD1306_I2C_ADDRESS, false);
}
*/

void ssd1306_move(uint8_t row, uint8_t col)
{
	if(col > 15) { col = 15; }
	if(row > 7) { row = 7; }

	ssd1306_move_raw(row, col << 3);
}

void ssd1306_clear(uint8_t value)
{
	uint16_t i;

	ssd1306_home();
	for(i = 1024; i > 0; i--) {
		ssd1306_data(value);
		}
}

#define Set_Lower_Column_Start_Address_CMD        0x00
#define Set_Higher_Column_Start_Address_CMD       0x10
#define Set_Page_Start_Address_CMD                0xB0
#define round                                     1
#define square                                    0

void ssd1306_gotopixelxy(unsigned char x_pos, unsigned char y_pos)
{
	ssd1306_command((Set_Page_Start_Address_CMD + y_pos));
	ssd1306_command(((x_pos & 0x0F) | Set_Lower_Column_Start_Address_CMD));
	ssd1306_command((((x_pos & 0xF0) >> 0x04) | Set_Higher_Column_Start_Address_CMD));
}

void ssd1306_set_pixel(unsigned char x_pos, unsigned char y_pos, unsigned char colour)
{
    unsigned char value=0x00,page=0x00,bit_pos=0x00;

    page = (y_pos / 8);
    bit_pos = (y_pos - (page * 8));
    value = pixelbuffer[((page * 132) + x_pos)];

    if((colour & 0x01) != 0)
    {
        value |= (1 << bit_pos);
    }
    else
    {
        value &= (~(1 << bit_pos));
    }

    pixelbuffer[((page * 132) + x_pos)] = value;
    ssd1306_gotopixelxy(x_pos, page);
    ssd1306_data(value);
}

void ssd1306_line(signed int x1, signed char y1, signed int x2, signed char y2, unsigned char colour)
{
    signed int dx = 0x0000;
    signed int dy = 0x0000;
    signed int stepx = 0x0000;
    signed int stepy = 0x0000;
    signed int fraction = 0x0000;

    dy = (y2 - y1);
    dx = (x2 - x1);

    if (dy < 0)
    {
        dy = -dy;
        stepy = -1;
    }
    else
    {
        stepy = 1;
    }

    if (dx < 0)
    {
        dx = -dx;
        stepx = -1;
    }
    else
    {
        stepx = 1;
    }

    dx <<= 1;
    dy <<= 1;

    ssd1306_set_pixel(x1, y1, colour);

    if (dx > dy)
    {
        fraction = (dy - (dx >> 1));
        while (x1 != x2)
        {
            if (fraction >= 0)
            {
                y1 += stepy;
                fraction -= dx;
            }
            x1 += stepx;
            fraction += dy;

            ssd1306_set_pixel(x1, y1, colour);
        }
    }
    else
    {
        fraction = (dx - (dy >> 1));
        while (y1 != y2)
        {
            if (fraction >= 0)
            {
                x1 += stepx;
                fraction -= dy;
            }
            y1 += stepy;
            fraction += dx;
            ssd1306_set_pixel(x1, y1, colour);
        }
    }
}

void ssd1306_fill(uint8_t row, uint8_t col, uint8_t count, uint8_t max, uint32_t pattern, int8_t pshift)
{
	uint8_t pstate = 0;

	ssd1306_move(row, col);
	for(max; max > 0; max--) {
		if(count > max) {
			if(pshift < 0) {
				ssd1306_data((pattern >> (pstate++ << 3)) & 0xFF);
				}
			else {
				ssd1306_data((pattern >> (pstate++ + pshift)) & 0xFF);
				}
			pstate = (pstate > 3) ? 0 : pstate;
			}
		else {
			ssd1306_data(0x00);
			}
		}
}

void ssd1306_put_tile(uint8_t *tile, uint8_t limit)
{
	uint16_t i;

	for(i = 0; i < limit; i++) {
		ssd1306_data(*tile++);
		}
}

void ssd1306_putc_raw(char c)
{
	uint16_t i;

	for(i = c << 3; i < (c << 3) + 8; i++) {
		ssd1306_data(font[i]);
		}
}

void ssd1306_putc(char c)
{
	// remap from petscii to ascii, shifts drawing characters into the lower 32 ascii cells
	if(c > 'A' && c < 'Z') { }               // upper-case ascii range
	else if(c >= 'a' && c <= 'z') { c -= 96; } // lower-case ascii range
	else if(c > 31 && c < 64) { }            // numbers and symbols
	else if(c < 32) { c += 96; }             // low ascii
	ssd1306_putc_raw(c);
}

// Draw a tile at an arbitrary pixel location (top, left) using an 8 byte tile buffer referenced by *tile.
// Slower than oled_putc(), potentially substantially so; only use for things that need
// finer grained positioning than is possible with tile cells, like sprites.
// Clips right and bottom edges properly; untested and not expected to work with negative positions.
void ssd1306_putxy(uint8_t left_pxl, uint8_t top_pxl, uint8_t *tile)
{
	uint8_t tcol;
	uint8_t tbuf[8], obuf[8];
	uint8_t top_cell = top_pxl >> 3;
	uint8_t left_cell = left_pxl >> 3;
	int8_t voff = top_pxl - ((top_cell << 3) - 1);
	int8_t hoff = left_pxl - ((left_cell << 3) - 1);

	if(voff == 0 && hoff == 0) {
		ssd1306_move(top_cell, left_pxl >> 3);
		ssd1306_put_tile(tile, 8);
		return;
		}
	else {
		for(tcol = 0; tcol < 8; tcol++) { // tile column
			tbuf[tcol] = (tile[tcol]) << ((uint8_t)voff); // shift left (down) by voff
			obuf[tcol] = (tile[tcol]) >> (8 - (uint8_t)voff); //shift right (up) by voff
			}

		ssd1306_move_raw(top_cell, left_pxl); // move_raw(row[0:7], column[0:127]) rows and pixels for extra confusion
		ssd1306_put_tile(&tbuf, (left_pxl > (SS1306_OLED_GEOM_W - 8)) ? 8 - hoff : 8);
		if(top_pxl < (SS1306_OLED_GEOM_H - 8)) {
			ssd1306_move_raw((top_cell + 1), left_pxl);
			ssd1306_put_tile(&obuf, (left_pxl > (SS1306_OLED_GEOM_W - 8)) ? 8 - hoff : 8);
			}
		}
}

void ssd1306_puts(char *str)
{
	while(*str != 0) {
		ssd1306_putc(*str++);
	}
}

// box graphics digit (a single digit, not byte or word; use this to render output of an int->bcd conversion etc)
void ssd1306_bigdigit(uint8_t top, uint8_t left, uint8_t num)
{
	uint8_t i;

//	const uint8_t chartable[] PROGMEM = { 0x10, 0x0E, 0x5D, 0x5D, 0x0D, 0x1D, // zero
	const uint8_t chartable[]  = {
		0x10, 0x0E, 0x5D, 0x5D, 0x0D, 0x1D, // zero
		0x20, 0x0E, 0x20, 0x5D, 0x20, 0x11, // one
		0x10, 0x0E, 0x10, 0x1D, 0x0D, 0x1D, // two
		0x10, 0x0E, 0x20, 0x13, 0x0D, 0x1D, // three
		0x5F, 0x5F, 0x0D, 0x13, 0x20, 0x5E, // four
		0x10, 0x0E, 0x0D, 0x0E, 0x0D, 0x1D, // five
		0x10, 0x0E, 0x0B, 0x0E, 0x0D, 0x1D, // six
		0x10, 0x0E, 0x20, 0x5B, 0x20, 0x5E, // seven
		0x10, 0x0E, 0x0B, 0x13, 0x0D, 0x1D, // eight
		0x10, 0x0E, 0x0d, 0x13, 0x0D, 0x1D  // nine
		};

	if(num > 9) { return; }
	ssd1306_move(top, left);
	for(i = 0; i < 6; i++) {
		if(i == 2 || i == 4) { ssd1306_move(++top, left); }
		ssd1306_putc(chartable[(num * 6) + i]);
	}
}

void ssd1306_box(uint8_t top, uint8_t left, uint8_t width, uint8_t height)
{
	uint8_t i;

	ssd1306_move(top, left);
	ssd1306_putc(BOX_TL); for(i=0; i < width - 2; i++) { ssd1306_putc(BOX_HL);} ssd1306_putc(BOX_TR);
	for(i=top+1; i < top+height-1; i++) {
		ssd1306_move(i, left);
		ssd1306_putc(BOX_VL);
		ssd1306_move(i, left + width - 1);
		ssd1306_putc(BOX_VL);
		}
	ssd1306_move(top + height - 1, left);
	ssd1306_putc(BOX_BL); for(i=0; i < width - 2; i++) { ssd1306_putc(BOX_HL);} ssd1306_putc(BOX_BR);
}

void ssd1306_box_erase(uint8_t top, uint8_t left, uint8_t width, uint8_t height)
{
	uint8_t i;

	ssd1306_move(top, left);
	ssd1306_putc(BOX_TL_ERASE); for(i=0; i < width - 2; i++) { ssd1306_putc(BOX_HL_ERASE);} ssd1306_putc(BOX_TR_ERASE);
	for(i=top+1; i < top+height-1; i++) {
		ssd1306_move(i, left);
		ssd1306_putc(BOX_VL_ERASE);
		ssd1306_move(i, left + width - 1);
		ssd1306_putc(BOX_VL_ERASE);
		}
	ssd1306_move(top + height - 1, left);
	ssd1306_putc(BOX_BL_ERASE); for(i=0; i < width - 2; i++) { ssd1306_putc(BOX_HL_ERASE);} ssd1306_putc(BOX_BR_ERASE);
}

void ssd1306_printfs(int row, int col, char *inchar)
{
	ssd1306_move(row, col);
	ssd1306_puts(inchar);
}

int ssd1306_printf(int row, int col, const char *fmt, ...)
{
	va_list args;
	int i;
	char printbuffer[40];

	va_start (args, fmt);
	i = vsnprintf (printbuffer, 250, fmt, args);
	va_end (args);

	ssd1306_move(row, col);
	ssd1306_puts(printbuffer);

	return i;
}

// TBD - limit number of total digits
void ssd1306_putbigfloat(int row,int col,float num,int digits,int decimal)
{
	char outstrg[7];
	int i,j,k;

	//sprintf(outstrg,"%f",num);
	i = 0;
	while(outstrg[i]!='.') {
		ssd1306_bigdigit(row,col+i*2,outstrg[i]-0x30);
		i++;
		}
	k = col+i*2;
	j = 0;
	while((i+1)<strlen(outstrg)) {
		ssd1306_bigdigit(row,col+(i-1)*2+1,outstrg[i]-0x30);
		i++;
		if(++j>decimal) break;
		}
	ssd1306_move(row,k);
	ssd1306_putc(' ');
	ssd1306_move(row+1,k);
	ssd1306_putc(' ');
	ssd1306_move(row+2,k);
	ssd1306_putc('.');
}

// TBD - output select number of digits - ie shift right
void ssd1306_putbigdigit(int row,int col,int num,int digits)
{
	char outstrg[7];
	int i;

	//sprintf(outstrg,"%5d",num);
	//printf("\n\routstrg=%s\n\r",outstrg);
	for(i=0; i<strlen(outstrg); i++) ssd1306_bigdigit(row,col+i*2,outstrg[i]-0x30);
}

void ssd1306_rectangle(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char fill, unsigned char colour, unsigned char type)
{
     unsigned short i = 0x00;
     unsigned short xmin = 0x00;
     unsigned short xmax = 0x00;
     unsigned short ymin = 0x00;
     unsigned short ymax = 0x00;

     if(fill != 0)
     {
        if(x1 < x2)
        {
           xmin = x1;
           xmax = x2;
        }
        else
        {
           xmin = x2;
           xmax = x1;
        }

        if(y1 < y2)
        {
           ymin = y1;
           ymax = y2;
        }
        else
        {
           ymin = y2;
           ymax = y1;
        }

        for(; xmin <= xmax; ++xmin)
        {
           for(i = ymin; i <= ymax; ++i)
           {
        	   ssd1306_set_pixel(x1, y1, colour);
           }
         }
     }

     else
     {
        ssd1306_line(x1, y1, x2, y1, colour);
        ssd1306_line(x1, y2, x2, y2, colour);
        ssd1306_line(x1, y1, x1, y2, colour);
        ssd1306_line(x2, y1, x2, y2, colour);
     }

     if(type != square)
     {
    	 ssd1306_set_pixel(x1, y1, ~colour);
    	 ssd1306_set_pixel(x1, y2, ~colour);
    	 ssd1306_set_pixel(x2, y1, ~colour);
    	 ssd1306_set_pixel(x2, y2, ~colour);
     }
}

void ssd1306_circle(unsigned char xc, unsigned char yc, unsigned char radius, unsigned char fill, unsigned char colour)
{
   signed int a = 0x0000;
   signed int b = 0x0000;
   signed int P = 0x0000;

   b = radius;
   P = (1 - b);

   do
   {
        if(fill != 0)
        {
            ssd1306_line((xc - a), (yc + b), (xc + a), (yc + b), colour);
        	ssd1306_line((xc - a), (yc - b), (xc + a), (yc - b), colour);
        	ssd1306_line((xc - b), (yc + a), (xc + b), (yc + a), colour);
        	ssd1306_line((xc - b), (yc - a), (xc + b), (yc - a), colour);
        }
        else
        {
        	ssd1306_set_pixel((xc + a), (yc + b), colour);
        	ssd1306_set_pixel((xc + b), (yc + a), colour);
        	ssd1306_set_pixel((xc - a), (yc + b), colour);
        	ssd1306_set_pixel((xc - b), (yc + a), colour);
        	ssd1306_set_pixel((xc + b), (yc - a), colour);
        	ssd1306_set_pixel((xc + a), (yc - b), colour);
        	ssd1306_set_pixel((xc - a), (yc - b), colour);
        	ssd1306_set_pixel((xc - b), (yc - a), colour);
        }

        if(P < 0)
        {
           P += (3 + (2 * a++));
        }
        else
        {
           P += (5 + (2 * ((a++) - (b--))));
        }
    }while(a <= b);
}

void SSD1306_Test(void)
{
	char inchar,outstrg[20];
	int i;
	float x,y,z;

	//memset(display_buffer, 0X55, 1025); // tried other values
	//memset(display_buffer, 0X55, 1025); // tried other values

    //ssd1306_Init();

	ssd1306_clear(255);
	delay_ms(500);
	ssd1306_clear(0);

	ssd1306_printf(0,0,"PaellaM07");
	ssd1306_printf(1,0,"abcdefghijklmn");
	ssd1306_printf(2,0,"opqrstuvwxyz");
	ssd1306_printf(3,0,"ABCDEFGHIJKLMN");
	ssd1306_printf(4,0,"OPQRSTUVWXYZ");

	delay_ms(2000);
	ssd1306_clear(0);

	x = 1.23;
	i = 0;
	while((inchar=GetCharnw())!='e') {
		//ssd1306_putbigdigit(3,0,i++,4);
		ssd1306_putbigfloat(3,0,x,8,3);
		x += 0.02;
		ssd1306_circle(20+i, 15, 7, 0, 1);
		ssd1306_circle(20+i+1, 15, 7, 0, 1);
		ssd1306_circle(20+i, 15, 7, 0, 0);
		i++;
		if(i>10) {
			ssd1306_circle(20+i, 15, 7, 0, 0);
			i = 0;
			}
		}

	ssd1306_box(6, 0, 3, 2);
	//delay_ms(2000);
	//ssd1306_box_erase(6, 0, 3, 2);

	ssd1306_line(0, 0, 127, 63, 1);
	ssd1306_rectangle(0, 0, 127, 10, 0, 1, round);
	ssd1306_circle(20, 20, 7, 0, 1);

	ssd1306_set_pixel(10,10,1);
	ssd1306_set_pixel(12,10,1);
	ssd1306_set_pixel(10,12,1);
	ssd1306_set_pixel(12,12,1);

	x = 1.23;
	y = -2.34;
	z = 8.23;
	if(0) {
	while((inchar=GetCharnw())!='e') {
		ssd1306_printf(2,0,"x=%8.2f  ",x);
		ssd1306_printf(3,0,"y=%8.2f  ",y);
		ssd1306_printf(4,0,"z=%8.2f  ",z);
		x += 0.02;
		y -= 0.001;
		z += 0.15;
		delay_ms(50);
		}
		}
}

// Update the ST7735 Color TFT display or SSD1306 OLED display
// Depending on mix of sensors and Pico Type (A,C or D) display data accordingly
void Update_Display(void)
{
	Update_Display_MPU9150(NOUSBOUT);

#ifdef  ST7735LCD
	// tbd
#endif
}
